import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/clint_eastwood1267.dart';
import 'package:contentsquare/src/russell_crowe588.dart';
import 'package:contentsquare/src/steve_mcqueen1293.dart';
import 'package:contentsquare/src/brad_pitt1402.dart';
import 'package:contentsquare/src/colin_firth1444.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/jon_voight1297.dart';
import 'package:contentsquare/src/william_holden1300.dart';
import 'package:contentsquare/src/peter_finch1398.dart';
import 'package:contentsquare/src/spencer_tracy1406.dart';
import 'package:contentsquare/src/robert_downey_jr1421.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _spencer_tracy499();
}


class _spencer_tracy499
    extends State<ContentsquareWebViewTrackerBuilder> {
  
  late final jason_robards62 = daniel_day_lewis502(
    kirk_douglas61: jeff_bridges1068,
    joe_pesci: _steve_mcqueen699,
  );

  
  String? _steve_mcqueen699() {
    
    
    
    
    if (!mounted) {
      russell_crowe588.nicolas_cage47(gene_kelly491());
      return null;
    }

    
    
    
    
    
    
    final alan_arkin1069 = context as Element;

    
    
    
    
    
    
    
    
    
    
    final albert_finney1070 = alan_arkin1069.renderObject;
    if (albert_finney1070 == null) {
      russell_crowe588.nicolas_cage47(liam_neeson490());
      return null;
    }
    if (albert_finney1070 is! james_stewart501) {
      russell_crowe588.gregory_peck45(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${albert_finney1070.runtimeType}.\n'
        'This is an error, it should always be a $james_stewart501.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final john_travolta1071 = albert_finney1070.orson_welles232;
    if (john_travolta1071 == null) {
      russell_crowe588.nicolas_cage47(cary_grant489());
      return null;
    }

    final matt_damon830 = gary_cooper604(
      steve_mcqueen829: albert_finney1070,
      hugh_jackman1210: charles_chaplin338.george_c_scott601(
        tom_cruise238: john_travolta1071.center,
      ),
    );

    return matt_damon830.orson_welles33(
      don_cheadle41: (hitElementData) {
        return peter_finch594(hitElementData.michael_douglas235);
      },
      
      
      geoffrey_rush42: () => null,
    );
  }

  @override
  void dispose() {
    jason_robards62._colin_firth706();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _gregory_peck503(
      gary_cooper261: _kevin_spacey500(
        alan_rickman6: jason_robards62.alan_rickman6,
        gary_cooper261: widget.builder(context, jason_robards62),
      ),
    );
  }
}


class _kevin_spacey500
    extends SingleChildRenderObjectWidget {
  const _kevin_spacey500({
    Key? matthew_mcconaughey1052,
    required this.alan_rickman6,
    required Widget gary_cooper261,
  }) : super(key: matthew_mcconaughey1052, child: gary_cooper261);

  
  final Future<Map<String, dynamic>> Function() alan_rickman6;

  @override
  RenderObject createRenderObject(BuildContext context) {
    return james_stewart501(alan_rickman6: alan_rickman6);
  }
}


class james_stewart501 extends RenderProxyBox {
  
  james_stewart501({
    required this.alan_rickman6,
  });

  
  final Future<Map<String, dynamic>> Function() alan_rickman6;
}


class daniel_day_lewis502 implements ContentsquareWebViewTracker {
  
  daniel_day_lewis502({
    required this.kirk_douglas61,
    required this.joe_pesci,
  });

  
  final charles_chaplin498 kirk_douglas61;

  
  final String? Function() joe_pesci;

  
  ContentsquareWebViewController? _philip_seymour_hoffman1072;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _philip_seymour_hoffman1072 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    final steve_mcqueen1073 = _philip_seymour_hoffman1072;

    if (steve_mcqueen1073 == null) {
      return russell_crowe588.nicolas_cage47(
        marlon_brando484(
          don_cheadle1259: null,
          george_c_scott1260: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        ),
      );
    }

    
    final henry_fonda1074 = steve_mcqueen1073.isChannelAvailable;
    if (henry_fonda1074 != null) {
      final bool joe_pesci1075;
      try {
        joe_pesci1075 =
            await _clint_eastwood701(
          henry_fonda1074: henry_fonda1074,
        );
      } catch (error) {
        return russell_crowe588.nicolas_cage47(
          marlon_brando484(
            don_cheadle1259: await steve_mcqueen1073.currentUrl?.call(),
            george_c_scott1260: 'Unexpected error while executing JS code, check the '
                'registration of the javascript channels: $error',
          ),
        );
      }
      if (!joe_pesci1075) {
        return russell_crowe588.nicolas_cage47(
          marlon_brando484(
            don_cheadle1259: await steve_mcqueen1073.currentUrl?.call(),
            george_c_scott1260:
                'JavascriptChannels are not registered, did you forget to use '
                'tracker.javascriptChannels?',
          ),
        );
      }
    }

    try {
      await steve_mcqueen1073.evaluateJavascript(
        '''
${_robert_mitchum700(jim_carrey1144: steve_mcqueen1073.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''',
      );
    } catch (error) {
      return russell_crowe588.nicolas_cage47(
        marlon_brando484(
          don_cheadle1259: await steve_mcqueen1073.currentUrl?.call(),
          george_c_scott1260: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $error',
        ),
      );
    }

    return russell_crowe588.nicolas_cage47(
      robert_downey_jr485(
        don_cheadle1259: await steve_mcqueen1073.currentUrl?.call(),
      ),
    );
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return ContentsquareJavascriptEventEnum.values
        .map(
          (event) => don_cheadle483(
            anthony_hopkins68: robin_williams292.christoph_waltz,
            joaquin_phoenix67: event,
            joe_pesci: joe_pesci,
          ).robin_williams,
        )
        .toSet();
  }

  
  String _robert_mitchum700({
    required String Function(String peter_o_toole1142, String anthony_hopkins1143)
        jim_carrey1144,
  }) {
    return '''
window.CSJavascriptBridge = {
  
  ${ContentsquareJavascriptEventEnum.values.map(
              (event) => '${event.name}: ${don_cheadle483(
                anthony_hopkins68: robin_williams292.christoph_waltz,
                joaquin_phoenix67: event,
                joe_pesci: joe_pesci,
              ).christoph_waltz7(
                jim_carrey1144: jim_carrey1144,
              )}',
            ).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  
  Future<bool> _clint_eastwood701({
    required Future<bool> Function(String) henry_fonda1074,
  }) async {
    for (final robin_williams in javascriptChannels) {
      final michael_douglas1076 =
          await henry_fonda1074(robin_williams.name);

      if (!michael_douglas1076) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _george_c_scott1077 = IList<Stopwatch>();

  
  Stopwatch _sidney_poitier702() {
    final mel_gibson1078 = Stopwatch();
    _george_c_scott1077 = _george_c_scott1077.add(mel_gibson1078);
    return mel_gibson1078;
  }

  
  void _marlon_brando703(Stopwatch mel_gibson1078) {
    mel_gibson1078.stop();
    _george_c_scott1077 = _george_c_scott1077.remove(mel_gibson1078);
  }

  
  @visibleForTesting
  static String get robert_de_niro => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> alan_rickman6() async {
    final steve_mcqueen1073 = _philip_seymour_hoffman1072;
    if (steve_mcqueen1073 == null) {
      throw robin_williams482();
    }

    
    
    final mel_gibson1078 = _sidney_poitier702();
    mel_gibson1078.start();

    bool clark_gable704() {
      return mel_gibson1078.elapsed >=
          kirk_douglas61.spencer_tracy64;
    }

    
    
    
    
    bool cary_grant705() {
      return !mel_gibson1078.isRunning;
    }

    Map<String, dynamic>? jamie_foxx840;
    
    
    
    
    while (!(clark_gable704() || cary_grant705())) {
      final sean_connery1079 = await steve_mcqueen1073.evaluateJavascript(
        robert_de_niro,
      );
      final Object? gary_cooper1080;
      try {
        final anthony_quinn1081 = jsonDecode(sean_connery1079);
        if (anthony_quinn1081 is String) {
          
          
          
          gary_cooper1080 = jsonDecode(anthony_quinn1081);
        } else {
          gary_cooper1080 = anthony_quinn1081;
        }
      } on FormatException catch (e) {
        russell_crowe588.nicolas_cage47(
          christoph_waltz494(
            humphrey_bogart66: sean_connery1079,
            paul_newman: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final johnny_depp1082 = gary_cooper1080 == null ||
          gary_cooper1080 is num ||
          gary_cooper1080 is Map<String, dynamic>;
      if (!johnny_depp1082) {
        russell_crowe588.nicolas_cage47(
          tommy_lee_jones495(
            humphrey_bogart66: gary_cooper1080,
          ),
        );
      }

      if (gary_cooper1080 is Map<String, dynamic>) {
        jamie_foxx840 = gary_cooper1080;
        break;
      }

      await Future.delayed(
        kirk_douglas61.robert_duvall63,
      );
    }

    _marlon_brando703(mel_gibson1078);

    if (jamie_foxx840 == null) {
      throw gary_cooper480(
        al_pacino69: kirk_douglas61.spencer_tracy64,
      );
    }

    return jamie_foxx840;
  }

  
  void _colin_firth706() {
    _george_c_scott1077.forEach(_marlon_brando703);
  }
}


class _gregory_peck503 extends InheritedWidget {
  const _gregory_peck503({
    Key? matthew_mcconaughey1052,
    required Widget gary_cooper261,
  }) : super(key: matthew_mcconaughey1052, child: gary_cooper261);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension woody_allen541 on Element {
  
  bool get _matthew_mcconaughey707 {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get denzel_washington708 {
    final alan_arkin1083 =
        getElementForInheritedWidgetOfExactType<_gregory_peck503>();
    return alan_arkin1083 != null || _matthew_mcconaughey707;
  }
}
