import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/james_cagney1316.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatefulWidget {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  @override
  State<SessionReplayMaskingScope> createState() =>
      _dustin_hoffman376();
}

class _dustin_hoffman376 extends State<SessionReplayMaskingScope> {
  late final MaskingConfig _matthew_mcconaughey896;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    final orson_welles897 = russell_crowe377.will_smith29(
      context,
      laurence_olivier1164: true,
    );
    if (orson_welles897 != null) {
      _matthew_mcconaughey896 = widget.maskingConfig.merge(
        orson_welles897.henry_fonda202,
      );
    } else {
      _matthew_mcconaughey896 = widget.maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return russell_crowe377(
      henry_fonda202: _matthew_mcconaughey896,
      gary_cooper261: widget.child,
    );
  }
}


class russell_crowe377 extends InheritedWidget {
  
  final MaskingConfig henry_fonda202;

  
  const russell_crowe377({
    required this.henry_fonda202,
    Key? matthew_mcconaughey1052,
    required Widget gary_cooper261,
  }) : super(key: matthew_mcconaughey1052, child: gary_cooper261);

  
  static russell_crowe377? will_smith29(
    BuildContext tom_cruise1163, {
    required bool laurence_olivier1164,
  }) {
    return laurence_olivier1164
        ? tom_cruise1163.dependOnInheritedWidgetOfExactType<russell_crowe377>()
        : tom_cruise1163.clint_eastwood636<russell_crowe377>();
  }

  @override
  bool updateShouldNotify(russell_crowe377 oldWidget) {
    return oldWidget.henry_fonda202 != henry_fonda202;
  }
}

extension on BuildContext {
  
  T? clint_eastwood636<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<russell_crowe377>()
        ?.widget as T?;
  }
}
